import Serverless from "serverless";
import { DeploymentExtended } from "@azure/arm-resources/esm/models";
import { BaseService } from "./baseService";
import { ArmTemplateConfig } from "../models/serverless";
import { ArmDeployment, ArmTemplateType } from "../models/armTemplates";
export declare class ArmService extends BaseService {
    private resourceClient;
    constructor(serverless: Serverless);
    /**
     * Creates an ARM deployment from a well-known configuration (consumption, premium, ase)
     * @param type The well-known template type
     */
    createDeploymentFromType(type: ArmTemplateType | string): Promise<ArmDeployment>;
    /**
     * Creates an ARM deployment from the serverless custom yaml configuration
     * @param armTemplateConfig The serverless yaml ARM template configuration
     */
    createDeploymentFromConfig(armTemplateConfig: ArmTemplateConfig): Promise<ArmDeployment>;
    /**
     * Deploys the specified ARM template to Azure via REST service call
     * @param deployment The ARM template to deploy
     */
    deployTemplate(deployment: ArmDeployment): Promise<DeploymentExtended>;
    /**
     * Applies sls yaml environment variables into the appSettings section of the function app configuration
     * @param deployment The ARM deployment
     */
    private applyEnvironmentVariables;
}
