import Serverless from "serverless";
import { BaseService } from "./baseService";
/**
 * Wrapper for operations on Azure Blob Storage account
 */
export declare class AzureBlobStorageService extends BaseService {
    /**
     * Account URL for Azure Blob Storage account. Depends on `storageAccountName` being set in baseService
     */
    private accountUrl;
    constructor(serverless: Serverless, options: Serverless.Options);
    /**
     * Upload a file to Azure Blob Storage
     * @param path Path of file to upload
     * @param containerName Name of container in Azure Blob storage for upload
     * @param blobName Name of blob file created as a result of upload
     */
    uploadFile(path: string, containerName: string, blobName?: string): Promise<void>;
    /**
     * Delete a blob from Azure Blob Storage
     * @param containerName Name of container containing blob
     * @param blobName Blob to delete
     */
    deleteFile(containerName: string, blobName: string): Promise<void>;
    /**
     * Lists files in container
     * @param ext - Extension of files to filter on when retrieving files
     * from container
     */
    listFiles(containerName: string, ext?: string): Promise<string[]>;
    /**
     * Lists the containers within the Azure Blob Storage account
     */
    listContainers(): Promise<string[]>;
    /**
     * Creates container specified in Azure Cloud Storage options
     * @param containerName - Name of container to create
     */
    createContainer(containerName: string): Promise<void>;
    /**
     * Delete a container from Azure Blob Storage Account
     * @param containerName Name of container to delete
     */
    deleteContainer(containerName: string): Promise<void>;
    /**
     * Get ServiceURL object for Azure Blob Storage Account
     */
    private getServiceURL;
    /**
     * Get a ContainerURL object to perform operations on Azure Blob Storage container
     * @param containerName Name of container
     * @param serviceURL Previously created ServiceURL object (will create if undefined)
     */
    private getContainerURL;
    /**
     * Get a BlockBlobURL object to perform operations on Azure Blob Storage Blob
     * @param containerName Name of container containing blob
     * @param blobName Name of blob
     */
    private getBlockBlobURL;
}
