"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var guard_1 = require("./guard");
describe("Guard", function () {
    function methodWithRequiredName(name) {
        guard_1.Guard.empty(name);
    }
    function methodWithRequiredNameWithParam(name) {
        guard_1.Guard.empty(name, "name", "Name is required");
    }
    function methodWithRequiredObject(options) {
        guard_1.Guard.null(options);
    }
    function methodWithRequiredExpression(value) {
        guard_1.Guard.expression(value, function (num) { return num > 0 && num < 100; });
    }
    describe("empty", function () {
        it("throws error on null value", function () {
            expect(function () { return methodWithRequiredName(null); }).toThrowError();
        });
        it("throws error on empty value", function () {
            expect(function () { return methodWithRequiredName(""); }).toThrowError();
        });
        it("throw error on whitespace", function () {
            expect(function () { return methodWithRequiredName(" "); }).toThrowError();
        });
        it("does not throw error on valid value", function () {
            expect(function () { return methodWithRequiredName("valid"); }).not.toThrowError();
        });
        it("throws specific error message", function () {
            expect(function () { return methodWithRequiredNameWithParam(null); }).toThrowError("Name is required");
        });
    });
    describe("null", function () {
        it("throws error on null value", function () {
            expect(function () { return methodWithRequiredObject(null); }).toThrowError();
        });
        it("does not throw error on valid value", function () {
            expect(function () { return methodWithRequiredObject({}); }).not.toThrowError();
        });
    });
    describe("expression", function () {
        it("throws error on invalide value", function () {
            expect(function () { return methodWithRequiredExpression(0); }).toThrowError();
        });
        it("does not throw error on valid value", function () {
            expect(function () { return methodWithRequiredExpression(1); }).not.toThrowError();
        });
    });
});
