"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var mockFactory_1 = require("../test/mockFactory");
var utils_1 = require("./utils");
describe("utils", function () {
    var sls;
    beforeEach(function () {
        var slsConfig = {
            service: "My test service",
            provider: "azure",
            functions: mockFactory_1.MockFactory.createTestSlsFunctionConfig(),
        };
        sls = mockFactory_1.MockFactory.createTestServerless();
        Object.assign(sls.service, slsConfig);
    });
    it("resolves handler when handler code is outside function folders", function () {
        sls.service["functions"].hello.handler = "src/handlers/hello.handler";
        mockFactory_1.MockFactory.updateService(sls);
        var functions = sls.service.getAllFunctions();
        var metadata = utils_1.Utils.getFunctionMetaData(functions[0], sls);
        var expectedMetadata = {
            entryPoint: "handler",
            handlerPath: path_1.default.normalize("../src/handlers/hello.js"),
            params: expect.anything(),
        };
        expect(metadata).toEqual(expectedMetadata);
    });
    it("resolves handler when code is in function folder", function () {
        sls.service["functions"].hello.handler = "hello/index.handler";
        mockFactory_1.MockFactory.updateService(sls);
        var functions = sls.service.getAllFunctions();
        var metadata = utils_1.Utils.getFunctionMetaData(functions[0], sls);
        var expectedMetadata = {
            entryPoint: "handler",
            handlerPath: path_1.default.normalize("index.js"),
            params: expect.anything(),
        };
        expect(metadata).toEqual(expectedMetadata);
    });
    it("resolves handler when code is at the project root", function () {
        sls.service["functions"].hello.handler = "hello.handler";
        mockFactory_1.MockFactory.updateService(sls);
        var functions = sls.service.getAllFunctions();
        var metadata = utils_1.Utils.getFunctionMetaData(functions[0], sls);
        var expectedMetadata = {
            entryPoint: "handler",
            handlerPath: path_1.default.normalize("../hello.js"),
            params: expect.anything(),
        };
        expect(metadata).toEqual(expectedMetadata);
    });
});
