/// <reference types="jest" />
import Serverless from "serverless";
import Service from "serverless/classes/Service";
import { HttpOperationResponse } from "@azure/ms-rest-js";
import { AuthResponse } from "@azure/ms-rest-nodeauth";
import { TokenClientCredentials } from "@azure/ms-rest-nodeauth/dist/lib/credentials/tokenClientCredentials";
import { ServiceListContainersSegmentResponse } from "@azure/storage-blob/typings/lib/generated/lib/models";
import { AxiosResponse, AxiosRequestConfig } from "axios";
import { ServerlessAzureConfig } from "../models/serverless";
import { AzureServiceProvider, ServicePrincipalEnvVariables } from "../models/azureProvider";
import { ApiCorsPolicy, ApiManagementConfig } from "../models/apiManagement";
import { DeploymentsListByResourceGroupResponse } from "@azure/arm-resources/esm/models";
import { ArmResourceTemplate } from "../models/armTemplates";
import { FunctionEnvelope, Site } from "@azure/arm-appservice/esm/models";
import { ApiManagementServiceResource, ApiContract } from "@azure/arm-apimanagement/esm/models";
export declare class MockFactory {
    static createTestServerless(config?: any): Serverless;
    static createTestService(functions?: any): Service;
    static updateService(sls: Serverless): void;
    static createTestServerlessOptions(): Serverless.Options;
    static createTestArmSdkResponse<R>(model: any, statusCode: number): Promise<R>;
    static createTestAuthResponse(): AuthResponse;
    static createTestFunctions(functionCount?: number): any[];
    static createTestFunction(name?: string): {
        properties: {
            name: string;
            config: {
                bindings: any[];
            };
        };
    };
    static createTestAzureCredentials(): TokenClientCredentials;
    static createTestDeployments(count?: number): DeploymentsListByResourceGroupResponse;
    static createTestAxiosResponse<T>(config: AxiosRequestConfig, responseJson: T, statusCode?: number): Promise<AxiosResponse>;
    static createTestAzureContainers(count?: number): ServiceListContainersSegmentResponse;
    static createTestBlockBlobUrl(containerName: string, blobName: string): {
        containerName: string;
        blobName: string;
        delete: jest.Mock<any, any>;
    };
    static createTestAzureBlobItems(id?: number, count?: number): {
        segment: {
            blobItems: any[];
        };
    };
    static createTestAzureBlobItem(id?: number, index?: number, ext?: string): {
        name: string;
    };
    static createTestAzureClientResponse<T>(responseJson: T, statusCode?: number): Promise<HttpOperationResponse>;
    static createTestServerlessYml(asYaml?: boolean, functionMetadata?: any): ServerlessAzureConfig;
    static createTestApimConfig(): ApiManagementConfig;
    static createTestFunctionApimConfig(name: string): {
        apim: {
            operations: {
                method: string;
                urlTemplate: string;
                displayName: string;
            }[];
        };
    };
    static createTestFunctionMetadata(name: string): {
        "handler": string;
        "events": ({
            "http": boolean;
            "x-azure-settings": {
                "authLevel": string;
                direction?: undefined;
                name?: undefined;
            };
        } | {
            "http": boolean;
            "x-azure-settings": {
                "direction": string;
                "name": string;
                authLevel?: undefined;
            };
        })[];
    };
    static createTestFunctionEvents(): ({
        "http": boolean;
        "x-azure-settings": {
            "authLevel": string;
            direction?: undefined;
            name?: undefined;
        };
    } | {
        "http": boolean;
        "x-azure-settings": {
            "direction": string;
            "name": string;
            authLevel?: undefined;
        };
    })[];
    static createTestFunctionsResponse(functions?: any): any[];
    static createTestAzureServiceProvider(): AzureServiceProvider;
    static createTestServicePrincipalEnvVariables(): ServicePrincipalEnvVariables;
    static createTestVariables(): {
        azureCredentials: {
            tokenCache: {
                _entries: {
                    accessToken: string;
                }[];
            };
        };
        subscriptionId: string;
    };
    static createTestSite(name?: string): Site;
    static createTestFunctionEnvelope(name?: string): FunctionEnvelope;
    static createTestBindings(bindingCount?: number): any[];
    static createTestBinding(): {
        authLevel: string;
        type: string;
        direction: string;
        name: string;
    } | {
        type: string;
        direction: string;
        name: string;
        authLevel?: undefined;
    };
    static createTestHttpBinding(direction?: string): {
        authLevel: string;
        type: string;
        direction: string;
        name: string;
    } | {
        type: string;
        direction: string;
        name: string;
        authLevel?: undefined;
    };
    static createTestBindingsObject(name?: string): {
        scriptFile: string;
        entryPoint: string;
        disabled: boolean;
        bindings: ({
            authLevel: string;
            type: string;
            direction: string;
            name: string;
        } | {
            type: string;
            direction: string;
            name: string;
            authLevel?: undefined;
        })[];
    };
    static createTestSlsFunctionConfig(): {
        hello: {
            apim: {
                operations: {
                    method: string;
                    urlTemplate: string;
                    displayName: string;
                }[];
            };
            "handler": string;
            "events": ({
                "http": boolean;
                "x-azure-settings": {
                    "authLevel": string;
                    direction?: undefined;
                    name?: undefined;
                };
            } | {
                "http": boolean;
                "x-azure-settings": {
                    "direction": string;
                    "name": string;
                    authLevel?: undefined;
                };
            })[];
        };
        goodbye: {
            apim: {
                operations: {
                    method: string;
                    urlTemplate: string;
                    displayName: string;
                }[];
            };
            "handler": string;
            "events": ({
                "http": boolean;
                "x-azure-settings": {
                    "authLevel": string;
                    direction?: undefined;
                    name?: undefined;
                };
            } | {
                "http": boolean;
                "x-azure-settings": {
                    "direction": string;
                    "name": string;
                    authLevel?: undefined;
                };
            })[];
        };
    };
    static createTestApimService(): ApiManagementServiceResource;
    static createTestApimApi(): ApiContract;
    private static createTestUtils;
    /**
     * Create a mock "request" module factory used to mock request objects that support piping
     * @param response The expected HTTP response
     */
    static createTestMockRequestFactory(response?: any): jest.Mock<{
        on: jest.Mock<any, any>;
        once: jest.Mock<any, any>;
        emit: jest.Mock<any, any>;
        write: jest.Mock<any, any>;
        end: jest.Mock<any, any>;
    }, [any, any]>;
    static createTestMockApiCorsPolicy(): ApiCorsPolicy;
    static createTestArmTemplate(): ArmResourceTemplate;
    private static createTestCli;
    private static createTestPluginManager;
}
