"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var js_yaml_1 = __importDefault(require("js-yaml"));
var serverless_1 = __importDefault(require("serverless"));
var ms_rest_js_1 = require("@azure/ms-rest-js");
function getAttribute(object, prop, defaultValue) {
    if (object && object[prop]) {
        return object[prop];
    }
    return defaultValue;
}
var MockFactory = /** @class */ (function () {
    function MockFactory() {
    }
    MockFactory.createTestServerless = function (config) {
        var sls = new serverless_1.default(config);
        sls.utils = getAttribute(config, "utils", MockFactory.createTestUtils());
        sls.cli = getAttribute(config, "cli", MockFactory.createTestCli());
        sls.pluginManager = getAttribute(config, "pluginManager", MockFactory.createTestPluginManager());
        sls.variables = getAttribute(config, "variables", MockFactory.createTestVariables());
        sls.service = getAttribute(config, "service", MockFactory.createTestService());
        sls.config.servicePath = "";
        return sls;
    };
    MockFactory.createTestService = function (functions) {
        if (!functions) {
            functions = MockFactory.createTestSlsFunctionConfig();
        }
        var serviceName = "serviceName";
        return {
            getAllFunctions: jest.fn(function () { return Object.keys(functions); }),
            getFunction: jest.fn(function (name) { return functions[name]; }),
            getAllEventsInFunction: jest.fn(),
            getAllFunctionsNames: jest.fn(function () { return Object.keys(functions); }),
            getEventInFunction: jest.fn(),
            getServiceName: jest.fn(function () { return serviceName; }),
            load: jest.fn(),
            mergeResourceArrays: jest.fn(),
            setFunctionNames: jest.fn(),
            update: jest.fn(),
            validate: jest.fn(),
            custom: null,
            provider: MockFactory.createTestAzureServiceProvider(),
            service: serviceName,
            artifact: "app.zip",
            functions: functions
        };
    };
    MockFactory.updateService = function (sls) {
        sls.service = MockFactory.createTestService(sls.service["functions"]);
    };
    MockFactory.createTestServerlessOptions = function () {
        return {
            extraServicePath: null,
            function: null,
            noDeploy: null,
            region: null,
            stage: null,
            watch: null,
        };
    };
    MockFactory.createTestArmSdkResponse = function (model, statusCode) {
        var response = {
            headers: new ms_rest_js_1.HttpHeaders(),
            request: null,
            status: statusCode,
        };
        var result = __assign({}, model, { _response: response });
        return Promise.resolve(result);
    };
    MockFactory.createTestAuthResponse = function () {
        return {
            credentials: MockFactory.createTestVariables()
                .azureCredentials,
            subscriptions: [
                {
                    id: "azureSubId",
                },
            ],
        };
    };
    MockFactory.createTestFunctions = function (functionCount) {
        if (functionCount === void 0) { functionCount = 3; }
        var functions = [];
        for (var i = 0; i < functionCount; i++) {
            functions.push(MockFactory.createTestFunction("function" + (i + 1)));
        }
        return functions;
    };
    MockFactory.createTestFunction = function (name) {
        if (name === void 0) { name = "TestFunction"; }
        return {
            properties: {
                name: name,
                config: {
                    bindings: MockFactory.createTestBindings()
                }
            }
        };
    };
    MockFactory.createTestAzureCredentials = function () {
        var credentials = {
            getToken: jest.fn(function () {
                var token = {
                    tokenType: "Bearer",
                    accessToken: "ABC123",
                };
                return Promise.resolve(token);
            }),
            signRequest: jest.fn(function (resource) { return Promise.resolve(resource); }),
        };
        // TODO: Reduce usage on tokenCache._entries[0]
        credentials["tokenCache"] = {
            _entries: [{ accessToken: "ABC123" }]
        };
        return credentials;
    };
    MockFactory.createTestDeployments = function (count) {
        if (count === void 0) { count = 5; }
        var result = [];
        for (var i = 0; i < count; i++) {
            result.push({
                name: "deployment" + (i + 1),
                properties: {
                    timestamp: new Date(),
                }
            });
        }
        return result;
    };
    MockFactory.createTestAxiosResponse = function (config, responseJson, statusCode) {
        if (statusCode === void 0) { statusCode = 200; }
        var statusText;
        switch (statusCode) {
            case 200:
                statusText = "OK";
                break;
            case 404:
                statusText = "NotFound";
                break;
        }
        var response = {
            config: config,
            data: JSON.stringify(responseJson),
            headers: {
                "Content-Type": "application/json; charset=utf-8",
            },
            status: statusCode,
            statusText: statusText,
        };
        return Promise.resolve(response);
    };
    MockFactory.createTestAzureContainers = function (count) {
        if (count === void 0) { count = 5; }
        var result = [];
        for (var i = 0; i < count; i++) {
            result.push({
                name: "container" + i,
                blobs: MockFactory.createTestAzureBlobItems(i),
            });
        }
        return { containerItems: result };
    };
    MockFactory.createTestBlockBlobUrl = function (containerName, blobName) {
        return {
            containerName: containerName,
            blobName: blobName,
            delete: jest.fn(),
        };
    };
    MockFactory.createTestAzureBlobItems = function (id, count) {
        if (id === void 0) { id = 1; }
        if (count === void 0) { count = 5; }
        var result = [];
        for (var i = 0; i < count; i++) {
            result.push(MockFactory.createTestAzureBlobItem(id, i));
        }
        return { segment: { blobItems: result } };
    };
    MockFactory.createTestAzureBlobItem = function (id, index, ext) {
        if (id === void 0) { id = 1; }
        if (index === void 0) { index = 1; }
        if (ext === void 0) { ext = ".zip"; }
        return {
            name: "blob-" + id + "-" + index + ext
        };
    };
    MockFactory.createTestAzureClientResponse = function (responseJson, statusCode) {
        if (statusCode === void 0) { statusCode = 200; }
        var response = {
            request: new ms_rest_js_1.WebResource(),
            parsedBody: responseJson,
            bodyAsText: JSON.stringify(responseJson),
            headers: new ms_rest_js_1.HttpHeaders(),
            status: statusCode,
        };
        return Promise.resolve(response);
    };
    MockFactory.createTestServerlessYml = function (asYaml, functionMetadata) {
        if (asYaml === void 0) { asYaml = false; }
        var data = {
            provider: {
                name: "azure",
                region: "West US 2"
            },
            plugins: [
                "serverless-azure-functions"
            ],
            functions: functionMetadata || MockFactory.createTestSlsFunctionConfig(),
        };
        return (asYaml) ? js_yaml_1.default.dump(data) : data;
    };
    MockFactory.createTestApimConfig = function () {
        return {
            name: "test-apim-resource",
            api: {
                name: "test-apim-api1",
                subscriptionRequired: false,
                displayName: "API 1",
                description: "description of api 1",
                protocols: ["https"],
                path: "test-api1",
            },
        };
    };
    MockFactory.createTestFunctionApimConfig = function (name) {
        return {
            apim: {
                operations: [
                    {
                        method: "get",
                        urlTemplate: name,
                        displayName: name,
                    },
                ],
            },
        };
    };
    MockFactory.createTestFunctionMetadata = function (name) {
        return {
            "handler": name + ".handler",
            "events": MockFactory.createTestFunctionEvents(),
        };
    };
    MockFactory.createTestFunctionEvents = function () {
        return [
            {
                "http": true,
                "x-azure-settings": {
                    "authLevel": "anonymous"
                }
            },
            {
                "http": true,
                "x-azure-settings": {
                    "direction": "out",
                    "name": "res"
                }
            }
        ];
    };
    MockFactory.createTestFunctionsResponse = function (functions) {
        var e_1, _a;
        var result = [];
        functions = functions || MockFactory.createTestSlsFunctionConfig();
        try {
            for (var _b = __values(Object.keys(functions)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var name_1 = _c.value;
                result.push({ properties: MockFactory.createTestFunctionEnvelope(name_1) });
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return result;
    };
    MockFactory.createTestAzureServiceProvider = function () {
        return {
            resourceGroup: "myResourceGroup",
            deploymentName: "myDeploymentName",
        };
    };
    MockFactory.createTestServicePrincipalEnvVariables = function () {
        return {
            azureSubId: "azureSubId",
            azureServicePrincipalClientId: "azureServicePrincipalClientId",
            azureServicePrincipalPassword: "azureServicePrincipalPassword",
            azureServicePrincipalTenantId: "azureServicePrincipalTenantId",
        };
    };
    MockFactory.createTestVariables = function () {
        return {
            azureCredentials: {
                tokenCache: {
                    _entries: [
                        {
                            accessToken: "token"
                        }
                    ]
                }
            },
            subscriptionId: "azureSubId",
        };
    };
    MockFactory.createTestSite = function (name) {
        if (name === void 0) { name = "Test"; }
        return {
            id: "appId",
            name: name,
            location: "West US",
            defaultHostName: "myHostName.azurewebsites.net",
            enabledHostNames: [
                "myHostName.azurewebsites.net",
                "myHostName.scm.azurewebsites.net",
            ]
        };
    };
    MockFactory.createTestFunctionEnvelope = function (name) {
        if (name === void 0) { name = "TestFunction"; }
        return {
            name: name,
            config: {
                bindings: MockFactory.createTestBindings()
            }
        };
    };
    MockFactory.createTestBindings = function (bindingCount) {
        if (bindingCount === void 0) { bindingCount = 3; }
        var bindings = [];
        for (var i = 0; i < bindingCount; i++) {
            bindings.push(MockFactory.createTestBinding());
        }
        return bindings;
    };
    MockFactory.createTestBinding = function () {
        // Only supporting HTTP for now, could support others
        return MockFactory.createTestHttpBinding();
    };
    MockFactory.createTestHttpBinding = function (direction) {
        if (direction === void 0) { direction = "in"; }
        if (direction === "in") {
            return {
                authLevel: "anonymous",
                type: "httpTrigger",
                direction: direction,
                name: "req",
            };
        }
        else {
            return {
                type: "http",
                direction: direction,
                name: "res"
            };
        }
    };
    MockFactory.createTestBindingsObject = function (name) {
        if (name === void 0) { name = "index.js"; }
        return {
            scriptFile: name,
            entryPoint: "handler",
            disabled: false,
            bindings: [
                MockFactory.createTestHttpBinding("in"),
                MockFactory.createTestHttpBinding("out")
            ]
        };
    };
    MockFactory.createTestSlsFunctionConfig = function () {
        return {
            hello: __assign({}, MockFactory.createTestFunctionMetadata("hello"), MockFactory.createTestFunctionApimConfig("hello")),
            goodbye: __assign({}, MockFactory.createTestFunctionMetadata("goodbye"), MockFactory.createTestFunctionApimConfig("goodbye")),
        };
    };
    MockFactory.createTestApimService = function () {
        return {
            name: "APIM Service Instance",
            location: "West US",
            publisherName: "Somebody",
            publisherEmail: "somebody@example.com",
            sku: {
                capacity: 0,
                name: "Consumption",
            }
        };
    };
    MockFactory.createTestApimApi = function () {
        return {
            name: "Api1",
            path: "/api1",
        };
    };
    MockFactory.createTestUtils = function () {
        return {
            appendFileSync: jest.fn(),
            copyDirContentsSync: jest.fn(),
            dirExistsSync: jest.fn(function () { return false; }),
            fileExistsSync: jest.fn(function () { return false; }),
            findServicePath: jest.fn(),
            generateShortId: jest.fn(),
            getVersion: jest.fn(),
            logStat: jest.fn(),
            readFile: jest.fn(),
            readFileSync: jest.fn(function (filename) {
                if (filename === "serverless.yml") {
                    return MockFactory.createTestServerlessYml();
                }
            }),
            walkDirSync: jest.fn(),
            writeFile: jest.fn(),
            writeFileDir: jest.fn(),
            writeFileSync: jest.fn(),
        };
    };
    /**
     * Create a mock "request" module factory used to mock request objects that support piping
     * @param response The expected HTTP response
     */
    MockFactory.createTestMockRequestFactory = function (response) {
        if (response === void 0) { response = {}; }
        return jest.fn(function (options, callback) {
            setImmediate(function () { return callback(null, response); });
            // Required interface for .pipe()
            return {
                on: jest.fn(),
                once: jest.fn(),
                emit: jest.fn(),
                write: jest.fn(),
                end: jest.fn(),
            };
        });
    };
    MockFactory.createTestMockApiCorsPolicy = function () {
        return {
            allowCredentials: false,
            allowedOrigins: ["*"],
            allowedHeaders: ["*"],
            exposeHeaders: ["*"],
            allowedMethods: ["GET", "POST"],
        };
    };
    MockFactory.createTestArmTemplate = function () {
        return {
            "$schema": "https://schema.management.azure.com/schemas/2015-01-01/deploymentTemplate.json#",
            "contentVersion": "1.0.0.0",
            "parameters": {
                "param1": {
                    "defaultValue": "",
                    "type": "String"
                },
                "param2": {
                    "defaultValue": "",
                    "type": "String"
                },
            },
            "variables": {},
            "resources": []
        };
    };
    MockFactory.createTestCli = function () {
        return {
            log: jest.fn(),
        };
    };
    MockFactory.createTestPluginManager = function () {
        return {
            addPlugin: jest.fn(),
            cliCommands: null,
            cliOptions: null,
            commands: null,
            deprecatedEvents: null,
            hooks: null,
            loadAllPlugins: jest.fn(),
            loadCommand: jest.fn(),
            loadCommands: jest.fn(),
            loadCorePlugins: jest.fn(),
            loadPlugins: jest.fn(),
            loadServicePlugins: jest.fn(),
            plugins: null,
            serverless: null,
            setCliCommands: jest.fn(),
            setCliOptions: jest.fn(),
            spawn: jest.fn(),
        };
    };
    return MockFactory;
}());
exports.MockFactory = MockFactory;
